/**
 * Back 2 Browser Bytecode Translator Copyright (C) 2012 Jaroslav Tulach
 * <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. Look for COPYING file in the top folder. If not, see
 * http://opensource.org/licenses/GPL-2.0.
 */
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.apidesign.bck2brwsr.htmlpage;

import net.java.html.canvas.ImageData;
import org.apidesign.bck2brwsr.core.JavaScriptBody;

/**
 *
 * @author Anton Epple <toni.epple@eppleton.de>
 */
public class ImageDataWrapper implements ImageData {

    private double width, height = -1;
    private Object imageData;
    private Data data;

    public ImageDataWrapper(Object imageData) {
        this.imageData = imageData;
    }

    private Data getData() {
        if (data == null) {
            data = new Data(getDataImpl(imageData));
        }
        return data;
    }

    @JavaScriptBody(args = {"imageData"}, body = "return imageData.data")
    public native Object getDataImpl(Object imageData);

    public double getWidth() {
        if (width == -1) {
            width = getWidthImpl(imageData);
        }
        return width;
    }

    @JavaScriptBody(args = {"imageData"}, body = "return imagedata.width;")
    private static native double getWidthImpl(Object imageData);

    public double getHeight() {
        if (height == -1) {
            height = getHeightImpl(imageData);
        }
        return height;
    }

    @JavaScriptBody(args = {"imageData"}, body = "return imagedata.height;")
    private static native double getHeightImpl(Object imageData);

    Object object() {
        return imageData;
    }

    @Override
    public int getR(double x, double y) {
        double idx = (x + y * width) * 4;
        return getData().get(idx);
    }

    @Override
    public int getG(double x, double y) {
        double idx = (x + y * width) * 4;
        return getData().get(idx + 1);
    }

    @Override
    public int getB(double x, double y) {
        double idx = (x + y * width) * 4;
        return getData().get(idx + 2);
    }

    @Override
    public int getA(double x, double y) {
        double idx = (x + y * width) * 4;
        return getData().get(idx + 3);
    }

    @Override
    public void setR(double x, double y, int value) {
        double idx = (x + y * width) * 4;
        getData().set(idx, value);
    }

    @Override
    public void setG(double x, double y, int value) {
        double idx = (x + y * width) * 4;
        getData().set(idx + 1, value);
    }

    @Override
    public void setB(double x, double y, int value) {
        double idx = (x + y * width) * 4;
        getData().set(idx + 2, value);
    }

    @Override
    public void setA(double x, double y, int value) {
        double idx = (x + y * width) * 4;
        getData().set(idx + 3, value);
    }

    public static class Data {

        Object data;

        public Data(Object data) {
            this.data = data;
        }

        public int get(double index) {
            return getImpl(data, index);
        }

        public void set(double index, int value) {
            setImpl(data, index, value);
        }

        @JavaScriptBody(args = {"data", "index", "value"}, body = "data[index]=value;")
        private static native void setImpl(Object data, double index, int value);

        @JavaScriptBody(args = {"imagedata", "index"}, body = "return data[index];")
        private static native int getImpl(Object data, double index);
    }
}
